/*	
 *	matchHeight() :: maintains same height of elements in a row
 *
 *	Copyright by Lazaworx
 *	http://www.lazaworx.com
 *	Author: Laszlo Molnar
 *
 *	Dual licensed under the MIT and GPL licenses.
 *	- http://www.opensource.org/licenses/mit-license.php
 *	- http://www.gnu.org/copyleft/gpl.html
 *
 *	Usage: $(element).matchHeight(options);
 *	options:
 		delegate		// the watched element for the row collection - default: 'card'
 		selector		// the items to match height - default: none = match cards 
 		lazyloadClass	// the container of images (<a>) to be loaded later
 */

;(function($, $window, $document) {
	'use strict';
			
	$.fn.matchHeight = function(settings) {
		
		settings = $.extend({}, $.fn.matchHeight.defaults, settings);
		
		return $(this).each(function() {
		
			var self = 		$(this),							// root element
				cards = 	self.find('.' + settings.delegate),	// elements to watch in a row
				ns = 		self.data('lmh_ns'),				// event namespace
				resizeTimeout,									// resize timeout
			
				// Getting one row of cards
				// 	<div class="column">
				//		<div class="card preload|lazyload" style="height:NNpx">
				//			<a class="thumb"><img></a>
				
				getRow = function(el) {
					
					if (!el.hasClass(settings.delegate)) {
						return $();
					}
					
					var	len = cards.length,
						card = el.closest('.' + settings.delegate),
						n = cards.index(card),
						row = card,
						to = Math.floor(card.offset().top);
						
						for (var i = n - 1; i >= 0; i--) {
							if (Math.round(cards.eq(i).offset().top) !== to) {
								break;
							}
							if (!cards.eq(i).hasClass(settings.lazyloadClass)) { 
								row = row.add(cards.eq(i));
							}
						}
												
						for (var i = n + 1; i < len; i++) {
							if (Math.round(cards.eq(i).offset().top) !== to) {
								break;
							}
							if (!cards.eq(i).hasClass(settings.lazyloadClass)) { 
								row = row.add(cards.eq(i));
							}
						}
						
						return row;
					},
					
				matchRow = function(cards) {
						if (!cards) {
							return;
						}
						
						if (settings.match) {
							
							// Multiple elements to match
							
							var el;
							
							// Resetting
							for (var i = 0; i < settings.match.length; i++) {
								el = (settings.match[i] === settings.delegate)? cards : cards.find('.' + settings.match[i]);
								if (el.hasClass(settings.verticalMiddleClass)) {
									if (settings.usePadding) {
										el.css({
											height: 		'auto',
											paddingTop:		0
										});
									} else {
										el.css({
											height: 		'auto',
											marginTop:		0,
											marginBottom:	0
										});
									}
								} else {
									el.height('auto');
								}
							}
							
							if (cards.length > 1) {
								
								// Multi column
								
								var	h,
									mh;
									
								for (var i = 0; i < settings.match.length; i++) {
									el = (settings.match[i] === settings.delegate)? cards : cards.find('.' + settings.match[i]);
									mh = 0;
									
									el.each(function() {
										if ((h = $(this).height()) > mh) {
											mh = h;
										}
									});
									
									el.each(function() {
										var t = $(this);
										
										if (t.hasClass(settings.verticalMiddleClass)) {
											if ((h = t.height()) < mh) {
												if (settings.usePadding) {
													t.css({
														height: 		mh,
														paddingTop:		Math.floor((mh - h) / 2)
													});
												} else {
													t.css({
														height: 		'auto',
														marginTop:		Math.floor((mh - h) / 2),
														marginBottom:	Math.ceil((mh - h) / 2)
													});
												}
											} else {
												if (settings.usePadding) {
													t.css({
														height: 		'auto',
														paddingTop:		0
													});
												} else {
													t.css({
														height: 		'auto',
														marginTop:		0,
														marginBottom:	0
													});
												}
											}
										} else {
											t.height(mh);
										}
									});
								}
							}
							
						} else {
							
							// Single element to match
							
							cards.height('auto');
								
							if (cards.length > 1) {
								var h,
									mh;
									
								cards.each(function() {
									if ((h = $(this).height()) > mh) {
										mh = h;
									}
								});
								
								cards.height(mh);
							}
						}
					},
					
				refresh = function(el) {
						if (!self.is(':visible')) {
							return;
						}
						
						var card = el? el.closest('.' + settings.delegate) : null;
						
						if (card && card.length) {
							// Refreshing single row around el
							matchRow(getRow(card));
						} else {
							// Refreshing a set of elements or everything
							var items,
								lte,
								row = $(),
								to;
								
							if (!el || el.data('lmh_ns')) {
								// down
								items = self.find('.' + settings.delegate);
							} else {
								// up
								items = el.closest('.' + settings.delegate);
							}
							
							items.not('.' + settings.lazyloadClass).each(function(i) {
								card = $(this).height('auto');
								
								if (row.length) {
									if ((card.offset().top - lte.offset().top) > 2) {
										matchRow(row);
										lte = row = card;
									} else {
										row = row.add(card);
									}
								} else {
									lte = row = card;
								}
							});
							
							matchRow(row);
						}
					};
					
			if (ns) {
				$document.add($window).off('.' + ns);
			}
			
			self.data('lmh_ns', ns = 'lmh_' + Math.floor(Math.random() * 10000));
			
			$window.on('scroll.' + ns + ' resize.' + ns, function(e) {
					clearTimeout(resizeTimeout);
					resizeTimeout = setTimeout(refresh, 50);
					return true;
				});
			
			self.on('refresh', function(e, el) {
					refresh((typeof el === UNDEF)? $(e.target) : $(el));
				});
			
			/*
			//	:: set up a load event listener and adjust row
			var imgs = self.find('.' + settings.lazyloadClass + ':first-child img');
			
			if (imgs.length) {
				imgs.one('load.' + ns, function() {
						refresh($(this));
					});
			}
			*/
			
			// Preloaded images 
			//	:: adjust the rows containing preloaded images when all is loaded
			self.find('.' + settings.preloadClass + ':first-child img')
				.waitAllImg(function() {
						refresh();
					});
					
		});
	};
	
	$.fn.matchHeight.defaults = {
		delegate:				'card',
		lazyloadClass:			'lazyload',
		verticalMiddleClass:	'vertical-middle',
		preloadClass:			'preload',
		usePadding:				true					// Using padding or margin for aligning vertically middle
	};
	
})(jQuery, jQuery(window), jQuery(document));
